@@ -42,7 +42,7 @@ demo(const char *latin1_infinitive)
 	    {
 		if (j != 0)
 		    printf(", ");
-		printf(person_array[i][j]);
+		printf("%s", person_array[i][j]);
 	    }
 	    printf("\n");
 	}
