@@ -1,8 +1,8 @@
-/*  $Id: main-window.cpp,v 1.7 2007/02/18 03:48:09 sarrazip Exp $
+/*  $Id: main-window.cpp,v 1.13 2009/03/10 03:03:52 sarrazip Exp $
     main-window.cpp - Input and conjugation window
 
     verbiste - French conjugation system
-    Copyright (C) 2003-2006 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2003-2009 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
+#include <ctype.h>
 
 #include <libintl.h>
 
@@ -63,6 +64,31 @@ static GtkWidget *resultNotebook = NULL;
 static const gint SP = 2;  // default spacing for the GTK+ boxes
 
 
+/*  External functions to be provided by the application.
+    A GNOME app would have these functions call gnome_config_get_string(), etc.
+*/
+char *get_config_string(const char *path);
+void set_config_string(const char *path, const char *value);
+void sync_config();
+
+
+/*****************************************************************************/
+
+
+// Encapsulates a gchar pointer to memory that needs to be freed with g_free().
+class Catena
+{
+public:
+    Catena(gchar *p) : ptr(p) {}
+    ~Catena() { g_free(ptr); }
+    gchar *get() const { return ptr; }
+    gint atoi() const { return ptr == NULL ? INT_MIN : static_cast<gint>(::atoi(ptr)); }
+    bool startsWithDigit() const { return ptr != NULL && ::isdigit(ptr[0]); }
+private:
+    gchar *ptr;
+};
+
+
 /*****************************************************************************/
 
 
@@ -100,6 +126,34 @@ ResultPage::ResultPage()
 /*****************************************************************************/
 
 
+// All code that wants to make the program quit must use this function
+// instead of calling gtk_main_quit() directly.
+//
+static
+void
+quit()
+{
+    // Save the current size of the results window.
+    gint resultWinWidth, resultWinHeight;
+    gtk_window_get_size(GTK_WINDOW(resultWin), &resultWinWidth, &resultWinHeight);
+
+    stringstream ss;
+    ss << resultWinWidth;
+    set_config_string("Preferences/ResultWindowWidth", ss.str().c_str());
+    ss.str("");
+    ss << resultWinHeight;
+    set_config_string("Preferences/ResultWindowHeight", ss.str().c_str());
+
+    // Save the state of the Show Pronouns checkbox.
+    bool active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showPronounsCB));
+    set_config_string("Preferences/ShowPronouns", active ? "1" : "0");
+
+    sync_config();  // commit changes to file
+
+    gtk_main_quit();
+}
+
+
 static
 void
 hideOrQuit(GtkWidget *, gpointer)
@@ -107,7 +161,7 @@ hideOrQuit(GtkWidget *, gpointer)
     if (hideOnDelete)
 	gtk_widget_hide(resultWin);
     else
-	gtk_main_quit();
+	quit();
 }
 
 
@@ -159,7 +213,7 @@ showAbout()
     GdkPixbuf *logo = gdk_pixbuf_new_from_file(logoFilename.c_str(), NULL);
 
     string copyright =
-	string("Copyright (C) 2003-2006 Pierre Sarrazin http://sarrazip.com/\n")
+	string("Copyright (C) 2003-2009 Pierre Sarrazin http://sarrazip.com/\n")
 	+ _("Distributed under the GNU General Public License");
 
     GtkWidget *about = gtk_about_dialog_new();
@@ -197,7 +251,7 @@ showAbout()
 
 static
 gboolean
-onKeyPressInEntry(GtkWidget *entry, GdkEventKey *event, gpointer data)
+onKeyPressInEntry(GtkWidget *entry, GdkEventKey *event, gpointer /*data*/)
 {
     g_return_val_if_fail(event != NULL, TRUE);
 
@@ -257,14 +311,6 @@ esc(const string &s)
 }
 
 
-/*  External functions to be provided by the application.
-    A GNOME app would have these functions call gnome_config_get_string(), etc.
-*/
-char *get_config_string(const char *path);
-void set_config_string(const char *path, const char *value);
-void sync_config();
-
-
 static
 void
 onSaveButton(GtkWidget *, gpointer)
@@ -281,12 +327,11 @@ onSaveButton(GtkWidget *, gpointer)
 				GTK_FILE_CHOOSER(chooser),
 				true);
 
-    gchar *saveDir = get_config_string("Files/SaveDirectory");
-    if (saveDir != NULL)
+    Catena saveDir = get_config_string("Files/SaveDirectory");
+    if (saveDir.get() != NULL)
 	gtk_file_chooser_set_current_folder(
 				    GTK_FILE_CHOOSER(chooser),
-				    saveDir);
-    g_free(saveDir);
+				    saveDir.get());
 
     gtk_file_chooser_set_current_name(
 				GTK_FILE_CHOOSER(chooser),
@@ -296,16 +341,15 @@ onSaveButton(GtkWidget *, gpointer)
     string filename;
     if (!cancel)
     {
-	char *f = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
-	filename = f;
-	g_free(f);
+	Catena f = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
+	filename = f.get();
 
 	// Remember the chosen directory:
-	gchar *dir = gtk_file_chooser_get_current_folder(
-					    GTK_FILE_CHOOSER(chooser));
-	set_config_string("Files/SaveDirectory", dir);
-	g_free(dir);
-	sync_config();
+	Catena dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(chooser));
+	set_config_string("Files/SaveDirectory", dir.get());
+
+	// We do not call sync_config() because it will be done by quit()
+	// when the program quits.
     }
     gtk_widget_destroy(chooser);
     if (cancel)
@@ -333,6 +377,7 @@ onSaveButton(GtkWidget *, gpointer)
     html << "<body>\n";
 
 
+    assert(resultNotebook != NULL);
     GtkWidget *w;
     for (int i = 0; (w = gtk_notebook_get_nth_page(
 			GTK_NOTEBOOK(resultNotebook), i)) != NULL;
@@ -447,6 +492,7 @@ appendResultPage(const string &utf8LabelText)
     ResultPage *rp = new ResultPage();
     GtkWidget *label = newLabel("<b>" + utf8LabelText + "</b>" , FALSE);
 
+    assert(resultNotebook != NULL);
     gtk_notebook_append_page(GTK_NOTEBOOK(resultNotebook),
 				GTK_WIDGET(rp->notebookPage),
 				GTK_WIDGET(label));
@@ -470,6 +516,27 @@ onShowPronounsToggled(GtkToggleButton *, gpointer)
 }
 
 
+static
+bool
+getResultWindowSize(gint &width, gint &height)
+{
+    Catena w = get_config_string("Preferences/ResultWindowWidth");
+    Catena h = get_config_string("Preferences/ResultWindowHeight");
+    if (!w.startsWithDigit() || !h.startsWithDigit())
+	return false;
+
+    enum { MIN = 0, MAX = 65535 };
+    width = w.atoi();
+    if (width < MIN || width > MAX)
+	return false;
+    height = h.atoi();
+    if (height < MIN || height > MAX)
+	return false;
+
+    return true;
+}
+
+
 static
 void
 showResultWin()
@@ -478,7 +545,13 @@ showResultWin()
     {
 	resultWin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(resultWin), PACKAGE_FULL_NAME);
-	gtk_window_set_default_size(GTK_WINDOW(resultWin), 380, 570);
+
+	// Get the size of the results window from the saved configuration, if available.
+	gint resultWinWidth, resultWinHeight;
+	if (!getResultWindowSize(resultWinWidth, resultWinHeight))  // if failed
+	    resultWinWidth = 610, resultWinHeight = 530;  // use default values
+	gtk_window_set_default_size(GTK_WINDOW(resultWin), resultWinWidth, resultWinHeight);
+
 	gtk_container_set_border_width(GTK_CONTAINER(resultWin), 4);
 
 	if (hideOnDelete)
@@ -492,7 +565,7 @@ showResultWin()
 	}
 	else
 	    g_signal_connect(G_OBJECT(resultWin), "delete_event",
-			G_CALLBACK(gtk_main_quit), NULL);
+			G_CALLBACK(quit), NULL);
 
 	/*
 	    Capture the key presses in order to hide or close the window
@@ -540,12 +613,16 @@ showResultWin()
 							FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(showPronounsCB), "toggled",
 				    G_CALLBACK(onShowPronounsToggled), NULL);
-
+	Catena showPronouns = get_config_string("Preferences/ShowPronouns");
+	if (showPronouns.get() != NULL)
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(showPronounsCB),
+					strcmp(showPronouns.get(), "1") == 0);
 
 	/*
 	    Create a notebook that receives the conjugations.
 	*/
 	resultNotebook = gtk_notebook_new();
+	assert(resultNotebook != NULL);
 	gtk_notebook_set_scrollable(GTK_NOTEBOOK(resultNotebook), TRUE);
 
 
@@ -580,6 +657,8 @@ showResultWin()
 	gtk_widget_show_all(GTK_WIDGET(resultWin));
     }
 
+    gtk_widget_grab_focus(GTK_WIDGET(verbEntry));
+
     gtk_window_present(GTK_WINDOW(resultWin));
 }
 
@@ -588,16 +667,19 @@ static
 void
 clearResultNotebook()
 {
-    GtkWidget *w;
-    while ((w = gtk_notebook_get_nth_page(
-			GTK_NOTEBOOK(resultNotebook), 0)) != NULL)
+    if (resultNotebook != NULL)
     {
-	ResultPage *rp = (ResultPage *) g_object_get_data(
-						G_OBJECT(w), "ResultPage");
-	if (rp == NULL)
-	    g_warning("clearResultNotebook: null ResultPage pointer");
-	gtk_notebook_remove_page(GTK_NOTEBOOK(resultNotebook), 0);
-	delete rp;
+	GtkWidget *w;
+	while ((w = gtk_notebook_get_nth_page(
+			    GTK_NOTEBOOK(resultNotebook), 0)) != NULL)
+	{
+	    ResultPage *rp = (ResultPage *) g_object_get_data(
+						    G_OBJECT(w), "ResultPage");
+	    if (rp == NULL)
+		g_warning("clearResultNotebook: null ResultPage pointer");
+	    gtk_notebook_remove_page(GTK_NOTEBOOK(resultNotebook), 0);
+	    delete rp;
+	}
     }
 
     gtk_widget_set_sensitive(GTK_WIDGET(saveButton), false);
@@ -608,10 +690,8 @@ static
 string
 tolowerUTF8(const string &s)
 {
-    gchar *down = g_utf8_strdown(s.data(), s.length());
-    string result = down;
-    g_free(down);
-    return result;
+    Catena down = g_utf8_strdown(s.data(), s.length());
+    return down.get();
 }
 
 
@@ -745,17 +825,20 @@ processText(const string &utf8UserText)
     gtk_widget_set_sensitive(GTK_WIDGET(saveButton), numPages > 0);
 
 
-    /*  PATCH: without this hack, a 2-page notebook has its first tab
-	active but showing the contents of the second page.  This can
-	be tested by entering "parais", which corresponds to the two
-	infinitives "paratre" and "parer".
-    */
-    if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(resultNotebook)) > 1)
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(resultNotebook), 1);
+    if (resultNotebook != NULL)
+    {
+	/*  PATCH: without this hack, a 2-page notebook has its first tab
+	    active but showing the contents of the second page.  This can
+	    be tested by entering "parais", which corresponds to the two
+	    infinitives "paratre" and "parer".
+	*/
+	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(resultNotebook)) > 1)
+	    gtk_notebook_set_current_page(GTK_NOTEBOOK(resultNotebook), 1);
 
 
 
-    gtk_notebook_set_current_page(GTK_NOTEBOOK(resultNotebook), 0);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(resultNotebook), 0);
+    }
 }
 
 
@@ -766,7 +849,7 @@ showErrorDialog(const string &msg)
 					GTK_DIALOG_MODAL,
 					GTK_MESSAGE_ERROR,
 					GTK_BUTTONS_CLOSE,
-					msg.c_str());
+					"%s", msg.c_str());
     gtk_dialog_run(GTK_DIALOG(dlg));
     gtk_widget_destroy(dlg);
 }
