@@ -1,4 +1,4 @@
-/*  $Id: FrenchVerbDictionary.cpp,v 1.20 2008/01/24 03:28:22 sarrazip Exp $
+/*  $Id: FrenchVerbDictionary.cpp,v 1.22 2009/03/10 02:59:36 sarrazip Exp $
     FrenchVerbDictionary.cpp - Dictionary of verbs and conjugation templates
 
     verbiste - French conjugation system
@@ -174,7 +174,7 @@ FrenchVerbDictionary::init(const string &conjugationFilename,
 
     {
 	for (int i = 0; i < 0xC0; i++)
-	    latin1TolowerTable[i] = tolower(char(i));
+	    latin1TolowerTable[i] = char(tolower(char(i)));
 	for (int i = 0xC0; i < 0xE0; i++)
 	    latin1TolowerTable[i] = char(i + 0x20);
 	for (int i = 0xE0; i < 0x100; i++)
@@ -350,7 +350,7 @@ FrenchVerbDictionary::readVerbs(xmlDocPtr doc) throw(logic_error)
 
     for (xmlNodePtr v = rootNodePtr->xmlChildrenNode; v != NULL; v = v->next)
     {
-	if (equal(v->name, "text"))
+	if (equal(v->name, "text") || equal(v->name, "comment"))
 	    continue;
 
 	xmlNodePtr i = v->xmlChildrenNode;
