@@ -1,4 +1,4 @@
-/*  $Id: Trie.h,v 1.7 2005/03/13 04:03:26 sarrazip Exp $
+/*  $Id: Trie.h,v 1.8 2009/03/10 02:59:36 sarrazip Exp $
     Trie.h - Tree structure for string storage
 
     verbiste - French conjugation system
@@ -88,9 +88,9 @@ public:
 	@param	index		length of the prefix
 	@param	userData	user data that is associated with the prefix
     */
-    virtual void onFoundPrefixWithUserData(const std::string &key,
-					std::string::size_type index,
-					const T *userData) const
+    virtual void onFoundPrefixWithUserData(const std::string &/*key*/,
+					std::string::size_type /*index*/,
+					const T * /*userData*/) const
 							throw()
     {
     }
