@@ -1,4 +1,4 @@
-/*  $Id: misc-types.cpp,v 1.5 2005/03/13 04:03:27 sarrazip Exp $
+/*  $Id: misc-types.cpp,v 1.6 2009/03/10 02:59:36 sarrazip Exp $
     misc-types.cpp - Miscellaneous types used by the dictionary class.
 
     verbiste - French conjugation system
@@ -68,13 +68,13 @@ ModeTensePersonNumber::set(const char *modeName,
     }
     else if (mode == PARTICIPLE_MODE)
     {
-	person = (personNum <= 2 ? 4 : 5);
+	person = static_cast<unsigned char>(personNum <= 2 ? 4 : 5);
 		// convention: 4=masculine, 5=feminie
 	plural = (personNum == 2 || personNum == 4);
     }
     else
     {
-	person = (personNum - 1) % 3 + 1;
+	person = static_cast<unsigned char>((personNum - 1) % 3 + 1);
 	plural = (personNum > 3);
     }
 }
