@@ -1,4 +1,4 @@
-/*  $Id: misc-types.h,v 1.3 2005/03/13 04:03:27 sarrazip Exp $
+/*  $Id: misc-types.h,v 1.4 2008/06/01 20:55:25 sarrazip Exp $
     misc-types.h - Miscellaneous types used by the dictionary class.
 
     verbiste - French conjugation system
@@ -24,11 +24,11 @@
 #define _H_misc_types
 
 #include <verbiste/c-api.h>
-#include <verbiste/HashMap.h>
 
 #include <assert.h>
 #include <vector>
 #include <string>
+#include <map>
 
 
 /**
@@ -150,46 +150,25 @@ typedef std::vector<std::string> PersonSpec;
 typedef std::vector<PersonSpec> TenseSpec;
 
 
-/** Function object that hashes a Tense value for use in a hash_map. */
-class TenseHasher
-{
-public:
-    size_t operator () (Tense t) const
-    {
-	return size_t(t) * 8191;  // multiply by a prime; result is < 32767
-    }
-};
-
-
 /**
     Mode specification.
     Contains tense specifications indexed by Tense values.
 */
-typedef HashMap<Tense, TenseSpec, TenseHasher> ModeSpec;
-
-
-/** Function object that hashes a Mode value for use in a hash_map. */
-struct ModeHasher
-{
-    size_t operator () (Mode m) const
-    {
-	return size_t(m) * 5449;  // multiply by a prime; result is < 32767
-    }
-};
+typedef std::map<Tense, TenseSpec> ModeSpec;
 
 
 /**
     Conjugation template specification.
     Contains mode specifications indexed by Mode values.
 */
-typedef HashMap<Mode, ModeSpec, ModeHasher> TemplateSpec;
+typedef std::map<Mode, ModeSpec> TemplateSpec;
 
 
 /**
     Conjugation system for the known verbs of a language.
     Contains conjugation templates indexed by template names.
 */
-typedef HashMap<std::string, TemplateSpec> ConjugationSystem;
+typedef std::map<std::string, TemplateSpec> ConjugationSystem;
 
 
 /**
@@ -198,7 +177,7 @@ typedef HashMap<std::string, TemplateSpec> ConjugationSystem;
     then a VerbTable would contain an entry where the key is "abaisser"
     and the value is "aim:er".
 */
-typedef HashMap<std::string, std::string> VerbTable;
+typedef std::map<std::string, std::string> VerbTable;
 
 
 /**
@@ -219,7 +198,7 @@ typedef HashMap<std::string, std::string> VerbTable;
     Thus, in this table, the key "e" would be be associated with a vector
     of five objects representing these five mode-tense-person combinations.
 */
-typedef HashMap<std::string, std::vector<ModeTensePersonNumber> >
+typedef std::map<std::string, std::vector<ModeTensePersonNumber> >
 					    TemplateInflectionTable;
 
 
@@ -230,7 +209,7 @@ typedef HashMap<std::string, std::vector<ModeTensePersonNumber> >
     accepted by the "aim:er" conjugation template (e.g., "e", "es",
     "ons", etc).
 */
-typedef HashMap<std::string, TemplateInflectionTable> InflectionTable;
+typedef std::map<std::string, TemplateInflectionTable> InflectionTable;
 
 
 /**
